# *
# *	Copyright (c) 2000 Alberto Reggiori / <alberto.reggiori@jrc.it>
# *	ISIS/RIT, Joint Research Center Ispra (I)
# *
# * NOTICE
# *
# * This product is distributed under a BSD/ASF like license as described in the 'LICENSE'
# * file you should have received together with this source code. If you did not get a
# * a copy of such a license agreement you can pick up one at:
# *
# *     http://xml.jrc.it/RDFStore/LICENSE
# *
# *
package RDF::Store::SetModel;
{
use RDF::Store::Model;
use RDF::SetModel;
use Carp;

@RDF::Store::SetModel::ISA = qw( RDF::Store::Model RDF::SetModel );

sub new {
        my ($pkg) = shift;
        bless $pkg->SUPER::new(@_), $pkg;
};                                

sub intersect {
	croak "Model ".$_[1]." is not instance of RDF::Model"
		unless( (defined $_[1]) ||
			($_[1]->isa('RDF::Model')) );

	# this operation should atomic (use a "monitor"/lock thingie)
	my $tmp = $_[0]->duplicate();

	my $k;
	my $v;
	while (($k,$v) = each %{$tmp->elements()}) {
		$_[0]->remove($v)
			if(!($_[1]->contains($t)));
	};
};

sub subtract {
	croak "Model ".$_[1]." is not instance of RDF::Model"
		unless( (defined $_[1]) &&
			($_[1]->isa('RDF::Model')) );

	# this operation should atomic (use a "monitor"/lock thingie)
	my $k;
	my $v;
	while (($k,$v) = each %{$_[1]->elements()}) {
		$_[0]->remove($v);
	};
};

sub unite {
	croak "Model ".$_[1]." is not instance of RDF::Model"
		unless( (defined $_[1]) &&
			($_[1]->isa('RDF::Model')) );

	# this operation should atomic (use a "monitor"/lock thingie)
	my $k;
	my $v;
	while (($k,$v) = each %{$_[1]->elements()}) {
		$_[0]->add($v);
	};
};

1;
};

__END__

=head1 NAME

RDF::Store::SetModel - implementation of the SetModel RDF API

=head1 SYNOPSIS

	use RDF::Store::SetModel;
        use RDF::Store::NodeFactory;
        use RDF::Store::FindIndex;
        use Data::MagicTie;

        my $factory= new RDF::Store::NodeFactory();
        my $index_db={};
        tie %{$index_db},"Data::MagicTie",'index/triples',(Q => 20);
        my $index=new RDF::Store::FindIndex($index_db);
	my $set = new RDF::Store::SetModel($factory,undef,$index,undef);

=head1 DESCRIPTION

An RDF::SetModel implementation using RDF::Store::Model and Digested URIs.

=head1 SEE ALSO

RDF::SetModel(3) RDF::Store::Model(3) Digest(3)

=head1 AUTHOR

	Alberto Reggiori <alberto.reggiori@jrc.it>
